<?php

namespace Enlightn\EnlightnPro\Analyzers\Security;

use Enlightn\Enlightn\Analyzers\Security\SecurityAnalyzer;
use Enlightn\Enlightn\Inspection\Reflector;
use Illuminate\Contracts\Http\Kernel;
use Illuminate\Http\Middleware\TrustHosts;
use Illuminate\Routing\Router;
use Enlightn\Enlightn\Analyzers\Concerns\AnalyzesMiddleware;
use Fideloper\Proxy\TrustProxies;

class HostInjectionAnalyzer extends SecurityAnalyzer
{
    use AnalyzesMiddleware;

    /**
     * The title describing the analyzer.
     *
     * @var string|null
     */
    public $title = 'Your application is protected against host injection attacks.';

    /**
     * The severity of the analyzer.
     *
     * @var string|null
     */
    public $severity = self::SEVERITY_MAJOR;

    /**
     * The time to fix in minutes.
     *
     * @var int|null
     */
    public $timeToFix = 5;

    /**
     * Create a new analyzer instance.
     *
     * @param  \Illuminate\Routing\Router  $router
     * @param  \Illuminate\Contracts\Http\Kernel  $kernel
     * @return void
     */
    public function __construct(Router $router, Kernel $kernel)
    {
        $this->router = $router;
        $this->kernel = $kernel;
    }

    /**
     * Get the error message describing the analyzer insights.
     *
     * @return string
     */
    public function errorMessage()
    {
        return 'Your application does not set trusted hosts while it is behind a reverse proxy or set of load '
            .'balancers. This exposes your application to Host Injection attacks. Laravel provides an '
            .'out-of-the-box middleware called "TrustHosts" to protect against such attacks. A very '
            .'simple fix could be to simply include the middleware in your web middleware group.';
    }

    /**
     * Execute the analyzer.
     *
     * @return void
     * @throws \ReflectionException
     */
    public function handle()
    {
        if (! $this->appUsesMiddleware(TrustHosts::class)) {
            $this->markFailed();
        }
    }

    /**
     * Determine whether to skip the analyzer.
     *
     * @return bool
     * @throws \ReflectionException|\Illuminate\Contracts\Container\BindingResolutionException
     */
    public function skip()
    {
        // Skip this analyzer if the app is not behind a trusted proxy.
        return ! $this->appUsesMiddleware(TrustProxies::class)
                || ! $this->trustedProxiesAreSetup();
    }

    /**
     * Determine whether trusted proxies are setup.
     *
     * @return bool
     * @throws \Illuminate\Contracts\Container\BindingResolutionException|\ReflectionException
     */
    protected function trustedProxiesAreSetup()
    {
        if (! empty(config('trustedproxy.proxies'))) {
            return true;
        }

        $middleware = app()->make(TrustProxies::class);
        $proxies = Reflector::get($middleware, 'proxies');

        return ! (empty($proxies) && is_null(config('trustedproxy.proxies')));
    }
}
